@@ -31,9 +31,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: script.py 4229 2008-09-15 23:00:50Z joanied $"
-__version__   = "$Revision: 4229 $"
-__date__      = "$Date: 2008-09-15 19:00:50 -0400 (Mon, 15 Sep 2008) $"
+__id__        = "$Id: script.py 4306 2008-10-25 17:48:21Z joanied $"
+__version__   = "$Revision: 4306 $"
+__date__      = "$Date: 2008-10-25 13:48:21 -0400 (Sat, 25 Oct 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1640,6 +1640,14 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source, True)
             return
 
+        # OOo Writer gets rather enthusiastic with focus: events for lists.
+        # See bug 546941.
+        #
+        if event.source.getRole() == pyatspi.ROLE_LIST \
+           and orca_state.locusOfFocus \
+           and self.isSameObject(orca_state.locusOfFocus.parent, event.source):
+            return
+
         default.Script.onFocus(self, event)
 
     def onActiveDescendantChanged(self, event):
@@ -2084,6 +2092,23 @@ class Script(default.Script):
             default.Script.getTextAttributes(self, acc, offset, get_defaults)
         return rv, start, end - 1
 
+    def getDisplayedText(self, obj):
+        """Returns the text being displayed for an object. Overridden here
+        because OpenOffice uses symbols (e.g. ">>" for buttons but exposes
+        more useful information via the accessible's name.
+
+        Arguments:
+        - obj: the object
+
+        Returns the text being displayed for an object or None if there isn't
+        any text being shown.
+        """
+
+        if obj.getRole() == pyatspi.ROLE_PUSH_BUTTON and obj.name:
+            return obj.name
+        else:
+            return default.Script.getDisplayedText(self, obj)
+
     def isFunctionalDialog(self, obj):
         """Returns true if the window is functioning as a dialog."""
 
